@@ -14,13 +14,13 @@ import contextlib
 import hashlib
 
 from oslo_serialization import jsonutils
+from oslo_utils import timeutils
 import six
 
 from keystonemiddleware.auth_token import _exceptions as exc
 from keystonemiddleware.auth_token import _memcache_crypt as memcache_crypt
 from keystonemiddleware.auth_token import _memcache_pool as memcache_pool
 from keystonemiddleware.i18n import _, _LE, _LW
-from keystonemiddleware.openstack.common import memorycache
 
 
 def _hash_key(key):
@@ -74,7 +74,11 @@ class _CachePool(list):
             c = self.pop()
         except IndexError:
             # the pool is empty, so we need to create a new client
-            c = memorycache.get_client(self._memcached_servers)
+            if self._memcached_servers:
+                import memcache
+                c = memcache.Client(self._memcached_servers, debug=0)
+            else:
+                c = _FakeClient()
 
         try:
             yield c
@@ -84,6 +88,7 @@ class _CachePool(list):
 
 class _MemcacheClientPool(object):
     """An advanced memcached client pool that is eventlet safe."""
+
     def __init__(self, memcache_servers, **kwargs):
         self._pool = memcache_pool.MemcacheClientPool(memcache_servers,
                                                       **kwargs)
@@ -103,15 +108,13 @@ class TokenCache(object):
 
     initialize() must be called before calling the other methods.
 
-    Store a valid token in the cache using store(); mark a token as invalid in
-    the cache using store_invalid().
+    Store data in the cache store.
 
     Check if a token is in the cache and retrieve it using get().
 
     """
 
     _CACHE_KEY_TEMPLATE = 'tokens/%s'
-    _INVALID_INDICATOR = 'invalid'
 
     def __init__(self, log, cache_time=None,
                  env_cache_name=None, memcached_servers=None,
@@ -144,16 +147,6 @@ class TokenCache(object):
         self._cache_pool = self._get_cache_pool(env.get(self._env_cache_name))
         self._initialized = True
 
-    def store(self, token_id, data):
-        """Put token data into the cache."""
-        self._LOG.debug('Storing token in cache')
-        self._cache_store(token_id, data)
-
-    def store_invalid(self, token_id):
-        """Store invalid token in cache."""
-        self._LOG.debug('Marking token as unauthorized in cache')
-        self._cache_store(token_id, self._INVALID_INDICATOR)
-
     def _get_cache_key(self, token_id):
         """Get a unique key for this token id.
 
@@ -212,7 +205,6 @@ class TokenCache(object):
         If token is invalid raise exc.InvalidToken
         return token only if fresh (not expired).
         """
-
         if not token_id:
             # Nothing to do
             return
@@ -229,32 +221,13 @@ class TokenCache(object):
             serialized = serialized.encode('utf8')
         data = self._deserialize(serialized, context)
 
-        # Note that _INVALID_INDICATOR and (data, expires) are the only
-        # valid types of serialized cache entries, so there is not
-        # a collision with jsonutils.loads(serialized) == None.
         if not isinstance(data, six.string_types):
             data = data.decode('utf-8')
-        cached = jsonutils.loads(data)
-        if cached == self._INVALID_INDICATOR:
-            self._LOG.debug('Cached Token is marked unauthorized')
-            raise exc.InvalidToken(_('Token authorization failed'))
 
-        # NOTE(jamielennox): Cached values used to be stored as a tuple of data
-        # and expiry time. They no longer are but we have to allow some time to
-        # transition the old format so if it's a tuple just return the data.
-        try:
-            data, expires = cached
-        except ValueError:
-            data = cached
+        return jsonutils.loads(data)
 
-        return data
-
-    def _cache_store(self, token_id, data):
-        """Store value into memcache.
-
-        data may be _INVALID_INDICATOR or a tuple like (data, expires)
-
-        """
+    def set(self, token_id, data):
+        """Store value into memcache."""
         data = jsonutils.dumps(data)
         if isinstance(data, six.text_type):
             data = data.encode('utf-8')
@@ -310,3 +283,52 @@ class SecureTokenCache(TokenCache):
 
     def _serialize(self, data, context):
         return memcache_crypt.protect_data(context, data)
+
+
+class _FakeClient(object):
+    """Replicates a tiny subset of memcached client interface."""
+
+    def __init__(self, *args, **kwargs):
+        # Ignores the passed in args
+        self.cache = {}
+
+    def get(self, key):
+        """Retrieve the value for a key or None.
+
+        This expunges expired keys during each get.
+        """
+        now = timeutils.utcnow_ts()
+        for k in list(self.cache):
+            (timeout, _value) = self.cache[k]
+            if timeout and now >= timeout:
+                del self.cache[k]
+
+        return self.cache.get(key, (0, None))[1]
+
+    def set(self, key, value, time=0, min_compress_len=0):
+        """Set the value for a key."""
+        timeout = 0
+        if time != 0:
+            timeout = timeutils.utcnow_ts() + time
+        self.cache[key] = (timeout, value)
+        return True
+
+    def add(self, key, value, time=0, min_compress_len=0):
+        """Set the value for a key if it doesn't exist."""
+        if self.get(key) is not None:
+            return False
+        return self.set(key, value, time, min_compress_len)
+
+    def incr(self, key, delta=1):
+        """Increment the value for a key."""
+        value = self.get(key)
+        if value is None:
+            return None
+        new_value = int(value) + delta
+        self.cache[key] = (self.cache[key][0], str(new_value))
+        return new_value
+
+    def delete(self, key, time=0):
+        """Delete the value associated with a key."""
+        if key in self.cache:
+            del self.cache[key]
